class AddProductButton {
  float m_screenX;
  float m_screenY;

  // Shape of the button
  void geometry() {
    fill( 255 );
    stroke( 77, 77, 77 );
    beginShape();
    vertex( -8, -2 );
    vertex( -2, -2 );
    vertex( -2, -8 );
    vertex( 2, -8 );
    vertex( 2, -2 );
    vertex( 8, -2 );
    vertex( 8, 2 );
    vertex( 2, 2 );
    vertex( 2, 8 );
    vertex( -2, 8 );
    vertex( -2, 2 );
    vertex( -8, 2 );
    endShape(CLOSE);
  }

  // Set the coordinates of the center of the button and draw the button
  void render( int _x, int _y ) {
    m_screenX = screenX( _x, _y );
    m_screenY = screenY( _x, _y );

    geometry();
  }
}

// Button to remove a product from the product list
class RemoveProductButton {

  float m_screenX;
  float m_screenY;

  // Set the center of the button in screen coordinates and draw the button
  void render( int _x, int _y ) {
    m_screenX = screenX( _x, _y );
    m_screenY = screenY( _x, _y );

    geometry();
  }

  // Shape of the button
  void geometry() {
    pushMatrix();
    rotate( radians( 45.0 ) );
    fill( 255 );
    stroke( 77, 77, 77 );
    beginShape();
    vertex( -8, -2 );
    vertex( -2, -2 );
    vertex( -2, -8 );
    vertex( 2, -8 );
    vertex( 2, -2 );
    vertex( 8, -2 );
    vertex( 8, 2 );
    vertex( 2, 2 );
    vertex( 2, 8 );
    vertex( -2, 8 );
    vertex( -2, 2 );
    vertex( -8, 2 );
    endShape(CLOSE);
    popMatrix();
  }
}



// Button to switch to the chart view
class ViewChartButton {
  // Center of the button in screen coordinates
  float m_screenX;
  float m_screenY;

  // Set the center of the button in screen coordinates and draw the button
  void render( int _x, int _y ) {
    m_screenX = screenX( _x, _y );
    m_screenY = screenY( _x, _y );

    fill( 255 );
    stroke( 77, 77, 77 );
    rectMode(CENTER);
    rect( 0, 0, 4, 16 );
    rect( 7, 2, 4, 12 );
    rect( 14, 4, 4, 8 ); 

    rectMode(CORNER);
  }
}


// Buttons to scroll through the list of products
class PageButton {
  float m_screenX;
  float m_screenY;
  boolean m_up;

  // Constructor
  public PageButton( boolean _up) {
    m_up = _up;
  }

  // Set the center of the button in screen coordinates and draw the button
  // Based on whether it is the up or down button
  void render( float _x, float _y ) {
    smooth();
    m_screenX = screenX( _x, _y );
    m_screenY = screenY( _x, _y );

    pushMatrix();
    if (m_up == false) rotate( radians( 180 ) );
    beginShape();
    vertex( -7, 4 );
    vertex( 0, -4 );
    vertex( 7, 4 );
    endShape(CLOSE);
    popMatrix();
    noSmooth();
  }
}


// Button for each product
class ChooseProductButton {

  // Characteristics of the button associated with the product
  int m_id;
  String m_productName;
  String m_servingSize;

  // UI measurements
  float m_screenX;
  float m_screenY;
  float m_buttonWidth;
  float m_buttonHeight;
  float m_buttonLeft;
  float m_buttonRight;
  float m_buttonTop;
  float m_buttonBottom;

  // Indentation of the text
  float m_indent;

  // Knowledge of selection
  boolean m_selected;
  // Position in the selected list
  int m_positionIndex;

  // Constructor
  // Inits are the characteristics, a position offscreen, and the dimensions of the button
  public ChooseProductButton( int _id, String _productName, String _servingSize, float _x, float _y, float _leftButtonWidth, float _leftButtonHeight ) {
    m_id = _id;
    m_productName = split(_productName, "(")[0];
    m_servingSize = _servingSize;    
    m_indent = 17.0;
    m_buttonWidth = _leftButtonWidth;
    m_buttonHeight = _leftButtonHeight;

    m_selected = false;
    m_positionIndex = -1;

    resetPosition( _x, _y );
  }

  // Draw the button at the supplied coordinates
  // The values passed in are the coordinates (0,0) in the current transformation matrix
  void renderButton( float _x, float _y ) {
    // Record the position of the button and change the coordinates to its current location 
    resetPosition( _x, _y );

    // Maximum length of product name in the menu
    int maxCharacters = 45;

    // Draw the name of the product
    textFont( abel18 );
    textAlign( LEFT );
    fill( 0 );
    noStroke();
    if ( m_productName.length() > maxCharacters )
      text( m_productName.substring( 0, maxCharacters ) + "...", m_buttonLeft + m_indent, m_buttonTop + m_buttonHeight / 2 );
    else 
      text( m_productName, m_buttonLeft + m_indent, m_buttonTop + m_buttonHeight / 2 );

    // Draw the serving size
    textFont( abel14 );
    text( m_servingSize + " per serving", m_buttonLeft + m_indent, m_buttonTop + m_buttonHeight / 2 + m_indent );

    stroke(77, 77, 77);
    noFill();
    line( m_buttonLeft, m_buttonTop + m_buttonHeight, m_buttonLeft + m_buttonWidth, m_buttonTop + m_buttonHeight );

    if ( makeChoice == false ) {
      // Color Codes for Product Selections
      for ( int i = 0; i < selectionsMenu.m_numProducts; i++ ) {
        fill( chart.m_colors[i] );
        noStroke();
        rect( selectionsMenu.m_chooseProductsPlaceX + selectionsMenu.m_chooseProductsWidth - 5, selectionsMenu.m_placeY + selectionsMenu.m_chooseProductsButtonsHeight * i + 1, 
        6, selectionsMenu.m_chooseProductsButtonsHeight - 1 );
      }
    }
    
    
    
  }

  // Record the current screen coordinates and change the position of the button
  void resetPosition( float _x, float _y ) {
    m_screenX = screenX( _x, _y );
    m_screenY = screenY( _x, _y );

    m_buttonLeft = m_screenX - m_buttonWidth / 2;
    m_buttonRight = m_screenX + m_buttonWidth / 2;
    m_buttonTop = m_screenY - m_buttonHeight / 2;
    m_buttonBottom = m_screenY + m_buttonHeight / 2;
  }
}


class SecondaryChooseButton {
  // Characteristics of the button associated with the product
  String m_description;
  int m_id;

  // UI measurements
  float m_screenX;
  float m_screenY;
  float m_buttonWidth;
  float m_buttonHeight;
  float m_buttonLeft;
  float m_buttonRight;
  float m_buttonTop;
  float m_buttonBottom;

  // Indentation of the text
  float m_indent;

  // Knowledge of selection
  boolean m_selected;
  // Position in the selected list
  int m_positionIndex;

  // Constructor
  // Inits are the characteristics, a position offscreen, and the dimensions of the button
  public SecondaryChooseButton( String _description, int _id, float _x, float _y, float _leftButtonWidth, float _leftButtonHeight, int _positionIndex) {
    m_description = _description;
    m_id = _id;
    m_indent = 10.0;
    m_buttonWidth = _leftButtonWidth;
    m_buttonHeight = _leftButtonHeight;

    m_selected = false;
    m_positionIndex = _positionIndex;

    resetPosition( _x, _y );
  }

  void renderButton( float _x, float _y ) {
    resetPosition( _x, _y );

    String descriptionShort = split( m_description, "(" )[0];

    if ( descriptionShort.length() > 40 ) {
      descriptionShort = descriptionShort.substring( 0, 40 );
    }
    else {
      descriptionShort = descriptionShort;
    }

    textFont( abel14 );
    fill( 0 );
    noStroke();
    textAlign( LEFT );

    text( descriptionShort, m_buttonLeft + m_indent, m_buttonTop + m_buttonHeight / 2 + 5 );
  }

  // Record the current screen coordinates and change the position of the button
  void resetPosition( float _x, float _y ) {
    m_screenX = screenX( _x, _y );
    m_screenY = screenY( _x, _y );

    m_buttonLeft = m_screenX - m_buttonWidth / 2;
    m_buttonRight = m_screenX + m_buttonWidth / 2;
    m_buttonTop = m_screenY - m_buttonHeight / 2;
    m_buttonBottom = m_screenY + m_buttonHeight / 2;
  }
}

